@@ -1,5 +1,7 @@
 """Sensor and mapping data from ecowitt."""
 from __future__ import annotations
+
+import datetime as dt
 from typing import Callable
 
 from dataclasses import dataclass, field
@@ -16,13 +18,16 @@ class EcoWittSensor:
     key: str
     stype: EcoWittSensorTypes
     station: EcoWittStation
-    value: None | str | int | float = field(default=None, init=False)
+    value: None | str | int | float | dt.datetime = field(default=None, init=False)
     last_update: float = field(default=0, init=False)
     last_update_m: float = field(default=0, init=False)
     update_cb: list[Callable[[None], None]] = field(default_factory=list, init=False)
 
     def update_value(
-        self, value: None | str | int | float, last_update: float, last_update_m: float
+        self,
+        value: None | str | int | float | dt.datetime,
+        last_update: float,
+        last_update_m: float,
     ) -> None:
         """Update the value of the sensor."""
         self.last_update = last_update
@@ -44,31 +49,32 @@ class EcoWittSensorTypes(enum.Enum):
     INTERNAL = 1
     PRESSURE_HPA = 2
     PRESSURE_INHG = 3
-    RATE_MM = 4
-    RATE_INCHES = 5
-    HUMIDITY = 6
-    DEGREE = 7
-    SPEED_KPH = 8
-    SPEED_MPH = 9
-    TEMPERATURE_C = 10
-    TEMPERATURE_F = 11
-    WATT_METERS_SQUARED = 12
-    UV_INDEX = 13
-    PM25 = 14
-    PM10 = 15
-    TIMESTAMP = 16
-    LIGHTNING_COUNT = 17
-    DISTANCE_KM = 18
-    DISTANCE_MILES = 19
-    LEAK = 20
-    VOLTAGE = 21
-    BATTERY_BINARY = 22
-    BATTERY_VOLTAGE = 23
-    BATTERY_PERCENTAGE = 24
-    LENGTH_INCHES = 25
-    LENGTH_MM = 26
+    RAIN_COUNT_MM = 4
+    RAIN_COUNT_INCHES = 5
+    RAIN_RATE_MM = 6
+    RAIN_RATE_INCHES = 7
+    HUMIDITY = 8
+    DEGREE = 9
+    SPEED_KPH = 10
+    SPEED_MPH = 11
+    TEMPERATURE_C = 12
+    TEMPERATURE_F = 13
+    WATT_METERS_SQUARED = 14
+    UV_INDEX = 15
+    PM25 = 16
+    PM10 = 17
+    TIMESTAMP = 18
+    LIGHTNING_COUNT = 19
+    LIGHTNING_DISTANCE_KM = 20
+    LIGHTNING_DISTANCE_MILES = 21
+    LEAK = 22
+    VOLTAGE = 23
+    BATTERY_BINARY = 24
+    BATTERY_VOLTAGE = 25
+    BATTERY_PERCENTAGE = 26
     CO2_PPM = 27
     LUX = 28
+    PERCENTAGE = 29
 
 
 @dataclass
@@ -84,24 +90,42 @@ SENSOR_MAP: dict[str, EcoWittMapping] = {
     "baromrelhpa": EcoWittMapping("Relative Pressure", EcoWittSensorTypes.PRESSURE_HPA),
     "baromabsin": EcoWittMapping("Absolute Pressure", EcoWittSensorTypes.PRESSURE_INHG),
     "baromrelin": EcoWittMapping("Relative Pressure", EcoWittSensorTypes.PRESSURE_INHG),
-    "rainratein": EcoWittMapping("Rain Rate", EcoWittSensorTypes.RATE_INCHES),
-    "eventrainin": EcoWittMapping("Event Rain Rate", EcoWittSensorTypes.RATE_INCHES),
-    "hourlyrainin": EcoWittMapping("Hourly Rain Rate", EcoWittSensorTypes.RATE_INCHES),
-    "totalrainin": EcoWittMapping("Total Rain", EcoWittSensorTypes.RATE_INCHES),
-    "dailyrainin": EcoWittMapping("Daily Rain Rate", EcoWittSensorTypes.RATE_INCHES),
-    "weeklyrainin": EcoWittMapping("Weekly Rain Rate", EcoWittSensorTypes.RATE_INCHES),
+    "rainratein": EcoWittMapping("Rain Rate", EcoWittSensorTypes.RAIN_RATE_INCHES),
+    "eventrainin": EcoWittMapping(
+        "Event Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
+    "hourlyrainin": EcoWittMapping(
+        "Hourly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
+    "totalrainin": EcoWittMapping("Total Rain", EcoWittSensorTypes.RAIN_COUNT_INCHES),
+    "dailyrainin": EcoWittMapping(
+        "Daily Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
+    "weeklyrainin": EcoWittMapping(
+        "Weekly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
     "monthlyrainin": EcoWittMapping(
-        "Monthly Rain Rate", EcoWittSensorTypes.RATE_INCHES
+        "Monthly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
+    "yearlyrainin": EcoWittMapping(
+        "Yearly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_INCHES
+    ),
+    "rainratemm": EcoWittMapping("Rain Rate", EcoWittSensorTypes.RAIN_RATE_MM),
+    "eventrainmm": EcoWittMapping("Event Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM),
+    "hourlyrainmm": EcoWittMapping(
+        "Hourly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM
+    ),
+    "totalrainmm": EcoWittMapping("Total Rain", EcoWittSensorTypes.RAIN_COUNT_MM),
+    "dailyrainmm": EcoWittMapping("Daily Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM),
+    "weeklyrainmm": EcoWittMapping(
+        "Weekly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM
+    ),
+    "monthlyrainmm": EcoWittMapping(
+        "Monthly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM
+    ),
+    "yearlyrainmm": EcoWittMapping(
+        "Yearly Rain Rate", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
-    "yearlyrainin": EcoWittMapping("Yearly Rain Rate", EcoWittSensorTypes.RATE_INCHES),
-    "rainratemm": EcoWittMapping("Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "eventrainmm": EcoWittMapping("Event Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "hourlyrainmm": EcoWittMapping("Hourly Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "totalrainmm": EcoWittMapping("Total Rain", EcoWittSensorTypes.RATE_MM),
-    "dailyrainmm": EcoWittMapping("Daily Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "weeklyrainmm": EcoWittMapping("Weekly Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "monthlyrainmm": EcoWittMapping("Monthly Rain Rate", EcoWittSensorTypes.RATE_MM),
-    "yearlyrainmm": EcoWittMapping("Yearly Rain Rate", EcoWittSensorTypes.RATE_MM),
     "humidity": EcoWittMapping("Humidity", EcoWittSensorTypes.HUMIDITY),
     "humidityin": EcoWittMapping("Indoor Humidity", EcoWittSensorTypes.HUMIDITY),
     "humidity1": EcoWittMapping("Humidity 1", EcoWittSensorTypes.HUMIDITY),
@@ -205,11 +229,11 @@ SENSOR_MAP: dict[str, EcoWittMapping] = {
         "Lightning strikes", EcoWittSensorTypes.LIGHTNING_COUNT
     ),
     "lightning": EcoWittMapping(
-        "Lightning strike distance", EcoWittSensorTypes.DISTANCE_KM
+        "Lightning strike distance", EcoWittSensorTypes.LIGHTNING_DISTANCE_KM
     ),
     "lightning_mi": EcoWittMapping(
         "Lightning strike distance",
-        EcoWittSensorTypes.DISTANCE_MILES,
+        EcoWittSensorTypes.LIGHTNING_DISTANCE_MILES,
     ),
     "tf_co2": EcoWittMapping("WH45 Temperature", EcoWittSensorTypes.TEMPERATURE_F),
     "tf_co2c": EcoWittMapping("WH45 Temperature", EcoWittSensorTypes.TEMPERATURE_C),
@@ -348,51 +372,81 @@ SENSOR_MAP: dict[str, EcoWittMapping] = {
     "tf_batt8": EcoWittMapping(
         "Soil Temperature 8 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
     ),
+    "leafwetness_ch1": EcoWittMapping("Leaf Wetness 1", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch2": EcoWittMapping("Leaf Wetness 2", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch3": EcoWittMapping("Leaf Wetness 3", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch4": EcoWittMapping("Leaf Wetness 4", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch5": EcoWittMapping("Leaf Wetness 5", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch6": EcoWittMapping("Leaf Wetness 6", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch7": EcoWittMapping("Leaf Wetness 7", EcoWittSensorTypes.PERCENTAGE),
+    "leafwetness_ch8": EcoWittMapping("Leaf Wetness 8", EcoWittSensorTypes.PERCENTAGE),
+    "leaf_batt1": EcoWittMapping(
+        "Leaf Wetness 1 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt2": EcoWittMapping(
+        "Leaf Wetness 2 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt3": EcoWittMapping(
+        "Leaf Wetness 3 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt4": EcoWittMapping(
+        "Leaf Wetness 4 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt5": EcoWittMapping(
+        "Leaf Wetness 5 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt6": EcoWittMapping(
+        "Leaf Wetness 6 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt7": EcoWittMapping(
+        "Leaf Wetness 7 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
+    "leaf_batt8": EcoWittMapping(
+        "Leaf Wetness 8 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE
+    ),
     "dateutc": EcoWittMapping("dateutc", EcoWittSensorTypes.INTERNAL),
     "fields": EcoWittMapping("field list", EcoWittSensorTypes.INTERNAL),
     "wh90batt": EcoWittMapping("WH90 Battery", EcoWittSensorTypes.BATTERY_VOLTAGE),
-    "wh90battpc": EcoWittMapping(
-        "WH90 Battery Percentage",
-        EcoWittSensorTypes.BATTERY_PERCENTAGE,
-    ),
     "ws90cap_volt": EcoWittMapping("WH90 Capacitor", EcoWittSensorTypes.VOLTAGE),
-    "rrain_piezo": EcoWittMapping("Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES),
+    "rrain_piezo": EcoWittMapping(
+        "Rain Rate Piezo", EcoWittSensorTypes.RAIN_RATE_INCHES
+    ),
     "erain_piezo": EcoWittMapping(
-        "Event Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Event Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
     "hrain_piezo": EcoWittMapping(
-        "Hourly Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Hourly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
     "drain_piezo": EcoWittMapping(
-        "Daily Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Daily Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
     "wrain_piezo": EcoWittMapping(
-        "Weekly Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Weekly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
     "mrain_piezo": EcoWittMapping(
-        "Monthly Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Monthly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
     "yrain_piezo": EcoWittMapping(
-        "Yearly Rain Rate Piezo", EcoWittSensorTypes.RATE_INCHES
+        "Yearly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_INCHES
     ),
-    "rrain_piezomm": EcoWittMapping("Rain Rate Piezo", EcoWittSensorTypes.RATE_MM),
+    "rrain_piezomm": EcoWittMapping("Rain Rate Piezo", EcoWittSensorTypes.RAIN_RATE_MM),
     "erain_piezomm": EcoWittMapping(
-        "Event Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Event Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "hrain_piezomm": EcoWittMapping(
-        "Hourly Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Hourly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "drain_piezomm": EcoWittMapping(
-        "Daily Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Daily Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "wrain_piezomm": EcoWittMapping(
-        "Weekly Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Weekly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "mrain_piezomm": EcoWittMapping(
-        "Monthly Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Monthly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "yrain_piezomm": EcoWittMapping(
-        "Yearly Rain Rate Piezo", EcoWittSensorTypes.RATE_MM
+        "Yearly Rain Rate Piezo", EcoWittSensorTypes.RAIN_COUNT_MM
     ),
     "runtime": EcoWittMapping("Runtime", EcoWittSensorTypes.INTERNAL),
 }
